#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from . config import PLAYER_COLORS
from . map_area import MapArea


class MapWindow:
    _HL_BG_COLOR = pygame.Color('black')

    def __init__(self, size, bg_color):
        self._bg_color = bg_color
        self._surface = pygame.Surface(size)

        self._map_pos = None
        self._map_scale = None
        self._map_areas = None

        self._dirty = False

    def blit(self, surface, rect, force):
        if self._dirty or force:
            surface.blit(self._surface, rect)
            self._dirty = False
            return True
        return False

    def init_grid(self, grid):
        self._surface.fill(self._bg_color)
        map_rect = pygame.Rect((0, 0), grid.map_size).fit(self._surface.get_rect())
        self._map_pos = (map_rect.x, map_rect.y)
        self._map_scale = min(map_rect.width / grid.map_size[0], map_rect.height / grid.map_size[1])
        self._map_areas = [
            MapArea(grid_area, grid.cells[grid_area.center], *self._map_pos, self._map_scale)
            for grid_area in grid.areas
        ]
        self._dirty = True

    def init_game(self, game):
        for area_idx, seat_idx in enumerate(game.area_seats):
            self._map_areas[area_idx].draw(
                self._surface, PLAYER_COLORS[seat_idx], num_dice=game.area_num_dice[area_idx]
            )
        self._dirty = True

    def draw_area(self, area_idx, player_idx, num_dice=None):
        self._map_areas[area_idx].draw(self._surface, PLAYER_COLORS[player_idx], num_dice=num_dice)
        self._dirty = True

    def highlight_area(self, area_idx, player_idx):
        self._map_areas[area_idx].draw(self._surface, self._HL_BG_COLOR, bd_color=PLAYER_COLORS[player_idx])
        self._dirty = True

    def get_map_pos(self, x, y):
        return (x - self._map_pos[0]) / self._map_scale, (y - self._map_pos[1]) / self._map_scale



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from . import config
from . import events
from . ctrl_view import CtrlView
from . widgets import Button, ToggleButton


class CtrlStart(CtrlView):
    def __init__(self, size, bg_color):
        super().__init__(size, bg_color)
        self._surface.fill(self._bg_color)

        self._num_seats = config.DEFAULT_NUM_PLAYERS

        w, h = size
        w_left, w_right = w - h, h

        rect = pygame.Rect(0, 0, w_left / (config.MAX_NUM_PLAYERS - 1) - 6, h)
        for num_seats in range(2, config.MAX_NUM_PLAYERS + 1):
            btn = ToggleButton(
                f'{num_seats}\nSEATS', rect,
                pygame.event.Event(events.BUTTON_NUM_SEATS, num_seats=num_seats),
                config.PLAYER_COLORS[num_seats - 1], on=num_seats <= self._num_seats
            )
            self._buttons.append(btn)
            btn.draw(self._surface)
            rect.x += rect.width + 6

        rect = pygame.Rect(w_left, 0, w_right, h / 2 - 3)
        btn = Button('SHUFFLE', rect, pygame.event.Event(events.BUTTON_SHUFFLE))
        self._buttons.append(btn)
        btn.draw(self._surface)
        rect.y = h - rect.height
        btn = Button('START', rect, pygame.event.Event(events.BUTTON_START))
        self._buttons.append(btn)
        btn.draw(self._surface)

    @property
    def num_seats(self):
        return self._num_seats

    @num_seats.setter
    def num_seats(self, num_seats):
        if num_seats < self._num_seats:
            range_ = range(num_seats - 1, self._num_seats - 1)
            on = False
        else:
            range_ = range(self._num_seats - 1, num_seats - 1)
            on = True
        for btn_idx in range_:
            self._buttons[btn_idx].set_on(on)
            self._buttons[btn_idx].draw(self._surface)
        self._num_seats = num_seats



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from . config import FONT_FILE_PATH


if not pygame.font:
    raise RuntimeError('pygame.font support not available')


class Text:
    SIZE_S = 20
    SIZE_M = 24
    SIZE_L = 42

    DEFAULT_COLOR = pygame.Color('black')

    _FONTS = {}

    @classmethod
    def _get_font(cls, size):
        font = cls._FONTS.get(size)
        if not font:
            font = pygame.font.Font(FONT_FILE_PATH, size)
            cls._FONTS[size] = font
        return font

    def __init__(self, text, size, rect, color=DEFAULT_COLOR):
        font = self._get_font(size)
        lines = text.split('\n')
        self._surface1 = font.render(lines[0], True, color)
        if len(lines) == 2:
            self._surface2 = font.render(lines[1], True, color)
            self._rect1 = self._surface1.get_rect(centerx=rect.centerx, bottom=rect.centery)
            self._rect2 = self._surface2.get_rect(centerx=rect.centerx, top=rect.centery)
        else:
            assert len(lines) == 1
            self._surface2 = None
            self._rect1 = self._surface1.get_rect(center=rect.center)

    def draw(self, surface):
        surface.blit(self._surface1, self._rect1)
        if self._surface2:
            surface.blit(self._surface2, self._rect2)

    def clear(self, surface, bg_color):
        pygame.draw.rect(surface, bg_color, self._rect1.union(self._rect2) if self._surface2 else self._rect1)


class Box:
    BD_WIDTH = 5
    DEFAULT_BG_COLOR = pygame.Color('white')
    DEFAULT_BD_COLOR = pygame.Color('black')

    def __init__(self, rect, bg_color=DEFAULT_BG_COLOR, bd_color=DEFAULT_BD_COLOR):
        self.rect = pygame.Rect(rect)
        self.bg_color = bg_color
        self.bd_color = bd_color

    def draw(self, surface, bg_color=None, bd_color=None):
        pygame.draw.rect(surface, bg_color if bg_color else self.bg_color, self.rect, border_radius=20)
        pygame.draw.rect(
            surface, bd_color if bd_color else self.bd_color, self.rect, width=self.BD_WIDTH, border_radius=20
        )


class Button:
    DOWN_FG_COLOR = pygame.Color('white')
    DOWN_BG_COLOR = Box.DEFAULT_BD_COLOR

    def __init__(self, text, rect, event):
        self._box = Box(rect)
        self._up_text = Text(text, Text.SIZE_S, rect)
        self._down_text = Text(text, Text.SIZE_S, rect, color=self.DOWN_FG_COLOR)
        self._event = event
        self._down = False

    @property
    def rect(self):
        return self._box.rect

    def draw(self, surface):
        if self._down:
            self._box.draw(surface, bg_color=self.DOWN_BG_COLOR)
            self._down_text.draw(surface)
        else:
            self._box.draw(surface)
            self._up_text.draw(surface)

    def mouse_down(self, x, y):
        assert not self._down
        if not self._box.rect.collidepoint(x, y):
            return False
        self._down = True
        return True

    def mouse_up(self, x, y):
        if self._box.rect.collidepoint(x, y) and self._down:
            self._down = False
            pygame.event.post(self._event)  # clicked
            return True
        assert not self._down
        return False

    def mouse_move(self, x, y):
        if self._box.rect.collidepoint(x, y):
            return False
        if self._down:
            self._down = False
            return True
        return False


class ToggleButton(Button):
    def __init__(self, text, rect, event, on_color, on=False):
        super().__init__(text, rect, event)
        self._on_color = on_color
        self.set_on(on)

    def set_on(self, on):
        self._box.bg_color = self._on_color if on else Box.DEFAULT_BG_COLOR


class EnableButton(Button):
    OFF_FG_COLOR = pygame.Color('gray')
    OFF_BD_COLOR = OFF_FG_COLOR

    def __init__(self, text, rect, event, on=True):
        super().__init__(text, rect, event)
        self._off_text = Text(text, Text.SIZE_S, rect, color=self.OFF_FG_COLOR)
        self._on = on

    def draw(self, surface):
        if self._on:
            super().draw(surface)
        else:
            self._box.draw(surface, bd_color=self.OFF_BD_COLOR)
            self._off_text.draw(surface)

    def mouse_down(self, x, y):
        return self._on and super().mouse_down(x, y)

    def mouse_up(self, x, y):
        return self._on and super().mouse_up(x, y)

    def mouse_move(self, x, y):
        return self._on and super().mouse_move(x, y)

    def set_on(self, on):
        self._on = on
        self._down = False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from . import config
from . import events
from . ctrl_view import CtrlView
from . widgets import Box, Text, EnableButton


class CtrlPlayers(CtrlView):
    def __init__(self, size, bg_color):
        super().__init__(size, bg_color)

        self._player_idx = -1

        w, h = size
        w_left, w_right = w - h, h

        rect = pygame.Rect(0, 0, 0, h)
        self._player_states = [
            _PlayerState(config.PLAYER_COLORS[player_idx], rect)
            for player_idx in range(config.MAX_NUM_PLAYERS)
        ]

        rect = pygame.Rect(w_left, 0, w_right, h)
        self._buttons.append(EnableButton('END\nTURN', rect, pygame.event.Event(events.BUTTON_END_TURN)))

    def init_match(self, match):
        self._surface.fill(self._bg_color)
        self._player_idx = match.player
        btn = self._buttons[0]
        x, w = 0, btn.rect.left / match.game.num_seats - 6
        for player_idx in match.game.seat_order:
            self._player_states[player_idx].init(
                self._surface, x, w, player_idx == self._player_idx, match.player_max_size[player_idx]
            )
            x += w + 6
        btn.set_on(self._player_idx == 0)
        btn.draw(self._surface)

    def set_current(self, player_idx, is_replay):
        assert 0 <= self._player_idx
        assert 0 <= player_idx
        self._player_states[self._player_idx].set_current(self._surface, False)
        self._player_states[player_idx].set_current(self._surface, True)
        if self._player_idx == 0 or player_idx == 0:
            self._buttons[0].set_on(player_idx == 0 and not is_replay)
            self._buttons[0].draw(self._surface)
        self._player_idx = player_idx

    def update(self, player_idx, max_size=None, num_stock=None):
        self._player_states[player_idx].update(self._surface, max_size, num_stock)


class _PlayerState:
    _SIZE_DY = -Text.SIZE_L * 0.25
    _STOCK_DY = Text.SIZE_S * 1.25
    _STOCK_0_COLOR = pygame.Color('gray')

    def __init__(self, color, rect):
        self._box = Box(rect, bd_color=color)
        self._size_text = None
        self._stock_text = None

    def init(self, surface, pos_x, width, current, max_size):
        self._box.rect.x = pos_x
        self._box.rect.width = width
        self._size_text = Text(str(max_size), Text.SIZE_L, self._box.rect.move(0, self._SIZE_DY))
        self._stock_text = Text('0', Text.SIZE_S, self._box.rect.move(0, self._STOCK_DY), color=self._STOCK_0_COLOR)
        self.set_current(surface, current)

    def set_current(self, surface, current):
        assert self._size_text  # not dead
        self._box.bg_color = self._box.bd_color if current else Box.DEFAULT_BG_COLOR
        self._box.draw(surface)
        self._size_text.draw(surface)
        self._stock_text.draw(surface)

    def update(self, surface, max_size=None, num_stock=None):
        assert self._size_text  # not dead
        self._size_text.clear(surface, self._box.bg_color)
        self._stock_text.clear(surface, self._box.bg_color)

        if max_size is not None:
            if max_size == 0:  # dead
                self._size_text = None
                self._stock_text = None
                return
            self._size_text = Text(str(max_size), Text.SIZE_L, self._box.rect.move(0, self._SIZE_DY))
        if num_stock is not None:
            self._stock_text = Text(
                str(num_stock), Text.SIZE_S, self._box.rect.move(0, self._STOCK_DY),
                color=self._STOCK_0_COLOR if num_stock == 0 else Text.DEFAULT_COLOR
            )

        self._size_text.draw(surface)
        self._stock_text.draw(surface)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from argparse import ArgumentParser

from pygame import Color
from dicewars.game import Game


APP_SIZE = (800, 800)

PLAYER_COLORS = [
    Color(color)
    for color in ('#B37FFE', '#B3FF01', '#009302', '#FF7FFE', '#FF7F01', '#B3FFFE', '#FFFF01', '#FF5858')
]

MAX_NUM_PLAYERS = len(PLAYER_COLORS)
DEFAULT_NUM_PLAYERS = Game.DEFAULT_NUM_SEATS

DEFAULT_STEP_INTERVAL_BASE = 5
STEP_INTERVAL = DEFAULT_STEP_INTERVAL_BASE * 100  # [ms]
SUPPLY_INTERVAL = DEFAULT_STEP_INTERVAL_BASE * 10  # [ms]

RESOURCES_PATH = Path(__file__).absolute().parent / 'resources'
FONT_FILE_PATH = RESOURCES_PATH / 'Anton-Regular.ttf'


def parse_cli_args():
    parser = ArgumentParser()
    parser.add_argument(
        '-i', '--interval', type=int,
        help=f'attack/supply step interval (min=1, default={DEFAULT_STEP_INTERVAL_BASE})'
    )
    args = parser.parse_args()

    if args.interval is not None:
        global STEP_INTERVAL, SUPPLY_INTERVAL
        interval_base = max(1, args.interval)
        STEP_INTERVAL = interval_base * 100
        SUPPLY_INTERVAL = interval_base * 10



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from dicewars.__version__ import __version__ as dicewars_version

from . __version__ import __version__
from . config import APP_SIZE, parse_cli_args
from . engine import Engine


def main():
    print(f'dicewars_pygame {__version__} (dicewars {dicewars_version})')
    pygame.init()
    pygame.display.set_caption(f'DiceWars (pygame) v{__version__}')

    screen = pygame.display.set_mode(APP_SIZE)
    clock = pygame.time.Clock()
    engine = Engine(screen)

    running = True
    while running:
        clock.tick(60)
        force_redraw = False

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
            elif event.type == pygame.WINDOWRESTORED:
                force_redraw = True
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                running = False
            elif event.type == pygame.MOUSEMOTION:
                engine.mouse_move(*event.pos)
            elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                engine.mouse_down(*event.pos)
            elif event.type == pygame.MOUSEBUTTONUP and event.button == 1:
                engine.mouse_up(*event.pos)
            elif pygame.USEREVENT <= event.type:
                engine.user_event(event)

        if engine.render(force_redraw):
            pygame.display.flip()

    pygame.quit()


def run():
    parse_cli_args()
    main()


if __name__ == '__main__':
    run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from . import events
from . ctrl_view import CtrlView
from . widgets import Box, Text, Button, EnableButton


class CtrlEnd(CtrlView):
    def __init__(self, size, bg_color):
        super().__init__(size, bg_color)
        self._surface.fill(self._bg_color)

        w, h = size

        rect = pygame.Rect(0, 0, h, h / 2 - 3)
        btn = EnableButton('CONTINUE', rect, pygame.event.Event(events.BUTTON_CONTINUE), on=False)
        self._buttons.append(btn)
        btn.draw(self._surface)
        rect.y = h - rect.height
        btn = Button('REPLAY', rect, pygame.event.Event(events.BUTTON_REPLAY))
        self._buttons.append(btn)
        btn.draw(self._surface)

        rect.right = w
        btn = Button('RESTART', rect, pygame.event.Event(events.BUTTON_RESTART))
        self._buttons.append(btn)
        btn.draw(self._surface)
        rect.y = 0
        btn = Button('SHUFFLE', rect, pygame.event.Event(events.BUTTON_SHUFFLE))
        self._buttons.append(btn)
        btn.draw(self._surface)

        self._box = Box((rect.width + 6, 0, w - (rect.width + 6) * 2, h))
        self._won_text = Text('CONGRATS, YOU WON!', Text.SIZE_L, self._box.rect)
        self._lost_text = Text('YOU LOST, TRY AGAIN?', Text.SIZE_L, self._box.rect)

    def show(self, winner_idx):
        btn = self._buttons[0]
        btn.set_on(winner_idx < 0)
        btn.draw(self._surface)

        self._box.draw(self._surface)
        if winner_idx == 0:
            self._won_text.draw(self._surface)
        else:
            self._lost_text.draw(self._surface)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from dicewars.grid import Grid
from dicewars.game import Game
from dicewars.match import Match, Attack
from dicewars.player import DefaultPlayer
from dicewars.util import pick_grid_area

from . import config
from . import events
from . map_window import MapWindow
from . ctrl_window import CtrlWindow


class Engine:
    _BG_COLOR = pygame.Color('white')

    def __init__(self, surface):
        self._surface = surface
        self._surface.fill(self._BG_COLOR)

        pad = 10
        w_surf, h_surf = self._surface.get_size()
        w_win, h_ctrl = w_surf - pad * 2, 100
        self._map_rect = pygame.Rect(pad, pad, w_win, h_surf - h_ctrl - pad * 3)
        self._map_window = MapWindow(self._map_rect.size, self._BG_COLOR)
        self._ctrl_rect = pygame.Rect(pad, self._map_rect.bottom + pad, w_win, h_ctrl)
        self._ctrl_window = CtrlWindow(self._ctrl_rect.size, self._BG_COLOR)

        self._grid = None
        self._game = None
        self._match = None
        self._ai_player = DefaultPlayer()
        self._timer_running = False

        self._init_grid()

    def mouse_down(self, x, y):
        if self._ctrl_rect.collidepoint(x, y):
            self._ctrl_window.mouse_down(x - self._ctrl_rect.x, y - self._ctrl_rect.y)
            return
        if self._timer_running:
            return
        if self._match and self._match.player == 0 and self._map_rect.collidepoint(x, y):
            map_pos = self._map_window.get_map_pos(x - self._map_rect.x, y - self._map_rect.y)
            grid_area = pick_grid_area(self._grid, *map_pos)
            if grid_area:
                self._set_user_area(grid_area.idx)

    def mouse_up(self, x, y):
        if self._ctrl_rect.collidepoint(x, y):
            self._ctrl_window.mouse_up(x - self._ctrl_rect.x, y - self._ctrl_rect.y)

    def mouse_move(self, x, y):
        self._ctrl_window.mouse_move(x - self._ctrl_rect.x, y - self._ctrl_rect.y)

    def user_event(self, event):
        if event.type == events.BUTTON_NUM_SEATS:
            self._ctrl_window.num_seats = event.num_seats
            self._init_game()
        elif event.type == events.BUTTON_SHUFFLE:
            self._init_grid()
        elif event.type == events.BUTTON_START:
            self._init_match()
        elif event.type == events.BUTTON_END_TURN:
            self._end_user_turn()
        elif event.type == events.BUTTON_CONTINUE:
            self._continue_match()
        elif event.type == events.BUTTON_RESTART:
            self._restart_match()
        elif event.type == events.BUTTON_REPLAY:
            self._replay_match()
        elif event.type == events.TIMER_NEXT_STEP:
            event.next_step()
        else:
            print('unhandled user event:', event)

    def render(self, force):
        dirty = self._map_window.blit(self._surface, self._map_rect, force)
        dirty |= self._ctrl_window.blit(self._surface, self._ctrl_rect, force)
        return dirty

    def _init_grid(self):
        self._grid = Grid()
        self._map_window.init_grid(self._grid)
        self._init_game()

    def _init_game(self):
        assert self._grid
        self._match = None
        self._game = Game(self._grid, num_seats=self._ctrl_window.num_seats)
        self._map_window.init_game(self._game)
        self._ctrl_window.show_start()

    def _init_match(self):
        assert self._game
        self._match = Match(self._game)
        self._ctrl_window.init_match(self._match)
        self._start_turn()

    def _continue_match(self):
        assert self._match and self._match.winner < 0
        self._ctrl_window.end_attack(self._match.last_attack)
        self._next_ai_action()

    def _restart_match(self):
        assert self._game
        self._map_window.init_game(self._game)
        self._init_match()

    def _start_turn(self):
        assert self._match
        self._ctrl_window.start_turn(self._match.player)
        if 0 < self._match.player:  # AI player
            self._next_ai_action()
        else:  # user player
            assert self._match.player == 0
            self._timer_running = False

    def _set_user_area(self, area_idx):
        assert self._match.player == 0
        if area_idx == self._match.from_area:  # unset from-area
            assert self._match.set_from_area(-1)
            self._map_window.draw_area(area_idx, 0)
            return
        if self._match.from_area < 0:  # (try to) set from-area
            if self._match.set_from_area(area_idx):
                self._map_window.highlight_area(area_idx, 0)
            return
        if self._match.set_to_area(area_idx):  # (try to) set to-area and attack
            self._map_window.highlight_area(area_idx, 0)
            self._attack()

    def _end_user_turn(self):
        assert self._match.player == 0
        if 0 <= self._match.from_area:  # unset from-area
            self._map_window.draw_area(self._match.from_area, 0)
        self._supply()

    def _next_ai_action(self):
        def show_from_area():
            self._map_window.highlight_area(self._match.from_area, self._match.player)
            self._start_timer(show_to_area, config.STEP_INTERVAL)

        def show_to_area():
            self._map_window.highlight_area(self._match.to_area, self._match.area_players[self._match.to_area])
            self._start_timer(self._attack, config.STEP_INTERVAL)

        attack_areas = self._ai_player.get_attack_areas(self._grid, self._match.state)
        if attack_areas:
            assert self._match.set_from_area(attack_areas[0])
            assert self._match.set_to_area(attack_areas[1])
            self._start_timer(show_from_area, config.STEP_INTERVAL)
        else:
            self._start_timer(self._supply, config.STEP_INTERVAL)

    def _attack(self):
        def show_from():
            self._ctrl_window.show_from_attack(self._match.last_attack)
            self._start_timer(show_to, config.STEP_INTERVAL)

        def show_to():
            self._ctrl_window.show_to_attack(self._match.last_attack)
            self._start_timer(end_attack, config.STEP_INTERVAL)

        def end_attack():
            la = self._match.last_attack
            self._map_window.draw_area(la.from_area, la.from_player, num_dice=la.from_area_num_dice)
            if la.victory:
                self._map_window.draw_area(la.to_area, la.from_player, num_dice=la.to_area_num_dice)
            else:
                self._map_window.draw_area(la.to_area, la.to_player)

            if 0 <= self._match.winner or (la.to_player == 0 and la.to_player_num_areas == 0):
                # match finished or user player just eliminated
                self._ctrl_window.show_end(self._match.winner)
                return
            self._ctrl_window.end_attack(la)

            if self._match.player == 0:  # user player
                self._timer_running = False
            else:  # AI player
                self._next_ai_action()

        assert self._match.attack()
        self._ctrl_window.start_attack(self._match.last_attack)
        self._start_timer(show_from, config.STEP_INTERVAL)

    def _supply(self):
        def next_supply():
            area_dice = self._ctrl_window.next_supply(self._match.last_supply)
            if area_dice:
                self._map_window.draw_area(area_dice[0], self._match.last_supply.player, area_dice[1])
                self._start_timer(next_supply, config.SUPPLY_INTERVAL)
            else:
                self._ctrl_window.end_supply(self._match.last_supply)
                self._start_turn()

        assert self._match.end_turn()
        self._ctrl_window.start_supply(self._match.last_supply)
        self._start_timer(next_supply, config.STEP_INTERVAL)

    def _replay_match(self):
        def next_step():
            nonlocal step_idx, player_idx, step_action
            step_idx += 1
            assert step_idx < self._match.num_steps
            step_action = self._match.history[step_idx]
            if isinstance(step_action, Attack):
                if player_idx != step_action.from_player:
                    player_idx = step_action.from_player
                    self._ctrl_window.start_turn(player_idx, is_replay=True)
                self._start_timer(show_from_area, config.STEP_INTERVAL)
            else:  # Supply
                if player_idx != step_action.player:
                    player_idx = step_action.player
                    self._ctrl_window.start_turn(player_idx, is_replay=True)
                self._start_timer(start_supply, config.STEP_INTERVAL)

        def show_from_area():
            self._map_window.highlight_area(step_action.from_area, step_action.from_player)
            self._start_timer(show_to_area, config.STEP_INTERVAL)

        def show_to_area():
            self._map_window.highlight_area(step_action.to_area, step_action.to_player)
            self._start_timer(start_attack, config.STEP_INTERVAL)

        def start_attack():
            self._ctrl_window.start_attack(step_action)
            self._start_timer(show_from_attack, config.STEP_INTERVAL)

        def show_from_attack():
            self._ctrl_window.show_from_attack(step_action)
            self._start_timer(show_to_attack, config.STEP_INTERVAL)

        def show_to_attack():
            self._ctrl_window.show_to_attack(step_action)
            self._start_timer(end_attack, config.STEP_INTERVAL)

        def end_attack():
            self._map_window.draw_area(
                step_action.from_area, step_action.from_player, num_dice=step_action.from_area_num_dice
            )
            if step_action.victory:
                self._map_window.draw_area(
                    step_action.to_area, step_action.from_player, num_dice=step_action.to_area_num_dice
                )
            else:
                self._map_window.draw_area(step_action.to_area, step_action.to_player)

            if step_idx < self._match.num_steps - 1:  # more actions available
                self._ctrl_window.end_attack(step_action)
                next_step()
            else:
                self._ctrl_window.show_end(self._match.winner)

        def start_supply():
            self._ctrl_window.start_supply(step_action)
            self._start_timer(next_supply, config.STEP_INTERVAL)

        def next_supply():
            area_dice = self._ctrl_window.next_supply(step_action)
            if area_dice:
                self._map_window.draw_area(area_dice[0], step_action.player, area_dice[1])
                self._start_timer(next_supply, config.SUPPLY_INTERVAL)
            else:
                self._ctrl_window.end_supply(step_action)
                next_step()  # NOTE: replay never starts after supply

        assert self._match and self._match.num_steps
        step_idx = -1
        player_idx = -1
        step_action = None
        self._map_window.init_game(self._game)
        self._ctrl_window.init_match(Match(self._game))
        next_step()

    def _start_timer(self, callback, timeout):
        pygame.time.set_timer(pygame.event.Event(events.TIMER_NEXT_STEP, next_step=callback), timeout, loops=1)
        self._timer_running = True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from dicewars.match import Match

from . import config
from . ctrl_view import CtrlView
from . widgets import Box, Text


class CtrlAttack(CtrlView):
    def __init__(self, size, bg_color):
        super().__init__(size, bg_color)

        rect = self._surface.get_rect()
        w_sum, h, dx = 80, rect.height, 10

        self._from_sum_box = Box((rect.centerx - w_sum - dx, 0, w_sum, h))
        w_val = self._from_sum_box.rect.left + Box.BD_WIDTH
        self._from_val_box = Box((0, 0, w_val, h))
        self._to_sum_box = Box((rect.centerx + dx, 0, w_sum, h))
        self._to_val_box = Box((self._to_sum_box.rect.right - Box.BD_WIDTH, 0, w_val, h))

        w_val /= (Match.AREA_MAX_NUM_DICE + 1)
        x = w_val * 0.5
        self._from_val_rects = []
        self._to_val_rects = []
        for _ in range(Match.AREA_MAX_NUM_DICE):
            self._from_val_rects.append(pygame.Rect(self._from_val_box.rect.right - w_val - x, 0, w_val, h))
            self._to_val_rects.append(pygame.Rect(self._to_val_box.rect.left + x, 0, w_val, h))
            x += w_val

        self._from_sum_text = None

    def init(self, from_player_idx, to_player_idx):
        self._surface.fill(self._bg_color)

        self._from_sum_box.bg_color = Box.DEFAULT_BG_COLOR
        self._from_sum_box.bd_color = config.PLAYER_COLORS[from_player_idx]
        self._from_val_box.bd_color = self._from_sum_box.bd_color
        self._from_sum_box.draw(self._surface)
        self._from_val_box.draw(self._surface)

        self._to_sum_box.bg_color = Box.DEFAULT_BG_COLOR
        self._to_sum_box.bd_color = config.PLAYER_COLORS[to_player_idx]
        self._to_val_box.bd_color = self._to_sum_box.bd_color
        self._to_sum_box.draw(self._surface)
        self._to_val_box.draw(self._surface)

    def show_from(self, dice, sum_dice):
        for dice_idx, dice_val in enumerate(dice):
            Text(str(dice_val), Text.SIZE_M, self._from_val_rects[dice_idx]).draw(self._surface)

        self._from_sum_text = Text(str(sum_dice), Text.SIZE_L, self._from_sum_box.rect)
        self._from_sum_text.draw(self._surface)

    def show_to(self, dice, sum_dice, victory):
        for dice_idx, dice_val in enumerate(dice):
            Text(str(dice_val), Text.SIZE_M, self._to_val_rects[dice_idx]).draw(self._surface)

        if victory:
            self._from_sum_box.bg_color = self._from_sum_box.bd_color
            self._from_sum_box.draw(self._surface)
            self._from_sum_text.draw(self._surface)
        else:
            self._to_sum_box.bg_color = self._to_sum_box.bd_color
            self._to_sum_box.draw(self._surface)
        Text(str(sum_dice), Text.SIZE_L, self._to_sum_box.rect).draw(self._surface)
        self._from_sum_text = None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

from . ctrl_start import CtrlStart
from . ctrl_players import CtrlPlayers
from . ctrl_attack import CtrlAttack
from . ctrl_supply import CtrlSupply
from . ctrl_end import CtrlEnd


class CtrlWindow:
    def __init__(self, size, bg_color):
        self._start = CtrlStart(size, bg_color)
        self._players = CtrlPlayers(size, bg_color)
        self._attack = CtrlAttack(size, bg_color)
        self._supply = CtrlSupply(size, bg_color)
        self._end = CtrlEnd(size, bg_color)
        self._supplies = []

        self._view = self._start
        self._dirty = True

    @property
    def num_seats(self):
        return self._start.num_seats

    @num_seats.setter
    def num_seats(self, num_seats):
        if self._start.num_seats != num_seats:
            self._start.num_seats = num_seats
            self._dirty = True

    def mouse_down(self, x, y):
        for btn in self._view.buttons:
            if btn.mouse_down(x, y):
                self._view.draw_button(btn)
                self._dirty = True
                break

    def mouse_up(self, x, y):
        for btn in self._view.buttons:
            if btn.mouse_up(x, y):
                self._view.draw_button(btn)
                self._dirty = True
                break

    def mouse_move(self, x, y):
        for btn in self._view.buttons:
            if btn.mouse_move(x, y):
                self._view.draw_button(btn)
                self._dirty = True
                break

    def blit(self, surface, rect, force):
        if self._dirty or force:
            surface.blit(self._view.surface, rect)
            self._dirty = False
            return True
        return False

    def show_start(self):
        if self._view is not self._start:
            self._view = self._start
            self._dirty = True

    def init_match(self, match):
        self._view = self._players
        self._players.init_match(match)
        self._dirty = True

    def start_turn(self, player_idx, is_replay=False):
        self._players.set_current(player_idx, is_replay)
        self._dirty = True

    def start_attack(self, attack):
        self._view = self._attack
        self._attack.init(attack.from_player, attack.to_player)
        self._dirty = True

    def show_from_attack(self, attack):
        self._attack.show_from(attack.from_dice, attack.from_sum_dice)
        self._dirty = True

    def show_to_attack(self, attack):
        self._attack.show_to(attack.to_dice, attack.to_sum_dice, attack.victory)
        self._dirty = True

    def end_attack(self, attack):
        self._view = self._players
        if attack.victory:
            self._players.update(attack.from_player, max_size=attack.from_player_max_size)
            self._players.update(attack.to_player, max_size=attack.to_player_max_size)
        self._dirty = True

    def start_supply(self, supply):
        assert not self._supplies
        self._view = self._supply
        for i, area_idx in enumerate(supply.areas):
            num_dice = supply.area_num_dice[i] - supply.dice[i] + 1
            self._supplies.extend((area_idx, num_dice + j) for j in range(supply.dice[i]))
        self._supplies.reverse()
        self._supply.init(supply.player, supply.sum_dice + supply.player_num_stock)
        self._dirty = True

    def next_supply(self, supply):
        if self._supplies:
            area_dice = self._supplies.pop()
            self._supply.hide(supply.player_num_stock + len(self._supplies))
            self._dirty = True
            return area_dice
        return None

    def end_supply(self, supply):
        self._view = self._players
        self._players.update(supply.player, num_stock=supply.player_num_stock)
        self._dirty = True

    def show_end(self, winner_idx):
        self._view = self._end
        self._end.show(winner_idx)
        self._dirty = True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from dicewars.match import Match

from . import config
from . ctrl_view import CtrlView
from . widgets import Box


class CtrlSupply(CtrlView):
    _DICE_BD_COLOR = pygame.Color('black')

    def __init__(self, size, bg_color):
        super().__init__(size, bg_color)
        self._surface.fill(self._bg_color)

        self._box = Box(self._surface.get_rect())

        size = self._box.rect.height // 4  # for box offsets
        size_ = size - 2  # for box drawing w/ gap
        dice_per_row = int(self._box.rect.width / size) - 1
        x, y = (self._box.rect.width - size * dice_per_row) * 0.5 + 1, size * 0.5 + 1
        rects0 = []  # 1st row
        rects1 = []  # 2nd row
        rects2 = []  # 3rd row
        for _ in range(dice_per_row):
            rects0.append(pygame.Rect(x, y, size_, size_))
            rects1.append(pygame.Rect(x, y + size, size_, size_))
            rects2.append(pygame.Rect(x, y + size + size, size_, size_))
            x += size
        self._dice_rects = rects0 + rects1 + rects2
        assert Match.PLAYER_MAX_NUM_STOCK <= len(self._dice_rects)

    def init(self, player_idx, num_dice):
        color = config.PLAYER_COLORS[player_idx]
        self._box.draw(self._surface, bd_color=color)

        for rect in self._dice_rects[:num_dice]:
            pygame.draw.rect(self._surface, color, rect, border_radius=5)
            pygame.draw.rect(self._surface, self._DICE_BD_COLOR, rect, width=2, border_radius=5)

    def hide(self, dice_idx):
        pygame.draw.rect(self._surface, self._bg_color, self._dice_rects[dice_idx])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from pathlib import Path


def load_version():
    version = {}
    with open(str(here / NAME / '__version__.py'), 'r') as fp:
        exec(fp.read(), version)
    return version['__version__']


def load_readme():
    with open(str(here / 'README.md'), 'r') as fp:
        readme = fp.read()
    return readme, 'text/markdown'


here = Path(__file__).absolute().parent

NAME = 'dicewars_pygame'
VERSION = load_version()
LONG_DESC, LONG_DESC_CONTENT_TYPE = load_readme()
CLASSIFIERS = [
    'Topic :: Games/Entertainment :: Board Games',
    'Intended Audience :: Education',
    'Programming Language :: Python :: 3 :: Only',
    'Development Status :: 4 - Beta',
    'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
]

PACKAGES = find_packages()
PACKAGE_DATA = {
    NAME: ['resources/Anton-Regular.*']
}
INSTALL_REQUIRES = [
    'pygame>=2.0.0',
    'dicewars==0.2.0',
]
ENTRY_POINTS = {
    'console_scripts': [f'{NAME}={NAME}.main:run']
}

setup(
    name=NAME,
    version=VERSION,
    description='DiceWars game GUI, based on pygame and dicewars',
    long_description=LONG_DESC,
    long_description_content_type=LONG_DESC_CONTENT_TYPE,
    author='Thomas Schott',
    author_email='scotty@c-base.org',
    url='https://github.com/scotty007/dicewars_pygame',
    classifiers=CLASSIFIERS,
    python_requires='>=3.7',
    packages=PACKAGES,
    package_data=PACKAGE_DATA,
    install_requires=INSTALL_REQUIRES,
    entry_points=ENTRY_POINTS,
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021 Thomas Schott <scotty@c-base.org>
#
# This file is part of dicewars_pygame.
#
# dicewars_pygame is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# dicewars_pygame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with dicewars_pygame.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from . widgets import Text


class MapArea:
    _BD_COLOR = pygame.Color('black')

    def __init__(self, grid_area, grid_center_cell, map_x0, map_y0, map_scale):
        self._points = [(map_x0 + x * map_scale, map_y0 + y * map_scale) for x, y in grid_area.border]
        # TODO: dice gfx
        bbox = [(map_x0 + x * map_scale, map_y0 + y * map_scale) for x, y in grid_center_cell.bbox]
        self._dice_rect = pygame.Rect(*bbox[0], bbox[1][0] - bbox[0][0], bbox[1][1] - bbox[0][1])
        self._num_dice = None

    def draw(self, surface, bg_color, bd_color=_BD_COLOR, num_dice=None):
        pygame.draw.polygon(surface, bg_color, self._points)
        pygame.draw.lines(surface, bd_color, True, self._points, 3)
        if num_dice is not None:
            self._num_dice = num_dice
        assert self._num_dice is not None
        Text(str(self._num_dice), Text.SIZE_S, self._dice_rect, color=bd_color).draw(surface)
